
#include <stdio.h>

#include "ez-list.h"

#include <assert.h>

struct node {
	struct ez_node ln;
	int value;
};

int scan_list(struct ez_list *list) {
	int count = 0;
	for (struct node *w=ez_list_head(list), *n=ez_node_succ(w);n;w=n,n=ez_node_succ(n)) {
		count++;
	}
	return count;
}

void node_remove(struct ez_node *n) {
	ez_node_remove(n);
}

void list_addtail(struct ez_list *list, struct ez_node *n) {
	ez_list_addtail(list, n);
}

static void check_list(struct ez_list *list, int max) {
	struct node *w, *n;
	int j;
	struct node *order[32];

	assert(list->tail == NULL);
	j = 0;
	for (w=ez_list_head(list), n=ez_node_succ(w);n;w=n,n=ez_node_succ(n)) {
		order[j++] = w;
		assert(j <= max);
	}
	for (w=ez_list_tail(list), n=ez_node_pred(w);n;w=n,n=ez_node_pred(n)) {
		assert(order[--j] == w);
		assert(j >= 0);
	}
	assert(j == 0);
}

static void check_16(struct ez_list *list) {
	struct node *w, *n;
	int j;

	check_list(list, 16);
	assert(ez_list_size(list) == 16);

	j = 0;
	for (w=ez_list_head(list), n=ez_node_succ(w);n;w=n,n=ez_node_succ(n)) {
		assert(ez_node_pred(n) == w);
		assert(j++ == w->value);
	}
	assert(j == 16);
}

int main(int argc, char **argv) {
	struct node nodes[16];
	struct ez_list list;
	
	for (int i=0;i<16;i++)
		nodes[i].value = i;

	ez_list_init(&list);
	for (int i=0;i<16;i++)
		ez_list_addtail(&list, &nodes[i]);
	check_16(&list);
	for (int i=0;i<16;i++) {
		struct node *n;
		n = ez_list_remtail(&list);
		assert(n->value == 15-i);
		check_list(&list, 16-i);
	}
	assert(ez_list_size(&list) == 0);
	assert(ez_list_empty(&list));

	for (int i=0;i<16;i++)
		ez_list_addhead(&list, &nodes[15-i]);
	check_16(&list);
	for (int i=0;i<16;i++) {
		struct node *n;
		n = ez_list_remhead(&list);
		assert(n->value == i);
		check_list(&list, 16-i);
	}
	assert(ez_list_size(&list) == 0);
	assert(ez_list_empty(&list));

	for (int i=0;i<16;i++)
		ez_list_addtail(&list, &nodes[i]);

	ez_node_remove(&nodes[15]);
	check_list(&list, 16);
	assert(ez_list_size(&list) == 15);

	ez_node_remove(&nodes[0]);
	check_list(&list, 16);
	assert(ez_list_size(&list) == 14);

	ez_node_remove(&nodes[7]);
	check_list(&list, 16);
	assert(ez_list_size(&list) == 13);

	ez_node_remove(&nodes[8]);
	check_list(&list, 16);
	assert(ez_list_size(&list) == 12);

	ez_list_insert(&list, &nodes[8], &nodes[9]);
	check_list(&list, 16);
	assert(ez_list_size(&list) == 13);

	ez_list_insert_after(&list, &nodes[7], &nodes[6]);
	check_list(&list, 16);
	assert(ez_list_size(&list) == 14);

	ez_list_insert(&list, &nodes[15], NULL);
	check_list(&list, 16);
	assert(ez_list_size(&list) == 15);

	ez_list_insert_after(&list, &nodes[0], NULL);
	check_list(&list, 16);
	check_16(&list);


	ez_list_init(&list);
	for (int i=0;i<16;i++)
		ez_list_addtail(&list, &nodes[i]);
	int j;
	j = 0;
	while (!ez_list_empty(&list)) {
		struct node *n = ez_list_head(&list);

		ez_node_remove(n);
		assert(n->value == j++);
		check_list(&list, 16);
	}
	assert(ez_list_size(&list) == 0);

	for (int i=0;i<16;i++)
		ez_list_addtail(&list, &nodes[i]);
	j = 16;
	while (!ez_list_empty(&list)) {
		struct node *n = ez_list_tail(&list);

		ez_node_remove(n);
		assert(n->value == --j);
		check_list(&list, 16);
	}

	
	return 0;
}
